import time
import unittest
import warnings

from vika import Vika

from . import TEST_API_BASE, TEST_API_TOKEN, TEST_TABLE


class TestUpdateRecords(unittest.TestCase):

    def setUp(self):
        warnings.simplefilter('ignore', ResourceWarning)
        vika = Vika(TEST_API_TOKEN)
        vika.set_api_base(TEST_API_BASE)
        self.dst = vika.datasheet(TEST_TABLE)

    def test_record_update(self):
        # 更新单个字段
        record = self.dst.records.get(title="无人生还")
        record.title = "无人生还2"
        self.assertEqual(record.title, "无人生还2")
        time.sleep(1 / 5)

        # 更新多个字段
        record = self.dst.records.get(title="无人生还2")
        r = record.update({"title": '无人生还3', "comment": '真好看'})
        self.assertEqual(r.title, "无人生还3")
        self.assertEqual(r.comment, "真好看")
        time.sleep(1 / 5)

        # 更新多条记录
        self.dst.records.filter(title="无人生还3").update(title="无人生还4")
        record = self.dst.records.get(title="无人生还4")
        self.assertEqual(record.title, "无人生还4")
        time.sleep(1 / 5)

    def test_bulk_update(self):
        records = self.dst.records.bulk_create([{
            "title": "魔法书1"
        }, {
            "title": "魔法书2"
        }, {
            "title": "魔法书3"
        }])
        update_data = [{
            "recordId": rec._id,
            "fields": {
                "title": 'new' + rec.title
            }
        } for rec in records]
        updated_records = self.dst.records.bulk_update(update_data)
        self.assertEqual(updated_records[0].title, 'new' + records[0].title)
        self.dst.delete_records(records)

    def test_update_or_create(self):
        # 更新
        record, created = self.dst.records.update_or_create(title="无人生还4",
                                                            defaults={
                                                                "title":
                                                                "无人生还",
                                                                "comment":
                                                                "真好看1"
                                                            })
        self.assertFalse(created)
        self.assertEqual(record.comment, "真好看1")
        time.sleep(1 / 5)
        # 创建记录
        record, created = self.dst.records.update_or_create(
            title="不能存在的记录2", defaults={"comment": "真好看"})
        self.assertTrue(created)
        self.assertEqual(record.comment, "真好看")
        self.assertEqual(record.title, "不能存在的记录2")
        self.dst.records.filter(title="不能存在的记录2").delete()


if __name__ == '__main__':
    unittest.main()



from typing import List

from vika.const import MAX_WRITE_RECORDS_PRE_REQ
from vika.datasheet.record import Record
from vika.exceptions import RecordDoesNotExist
from vika.types import RawRecord
from vika.utils import chunks, trans_key


class QuerySet:

    def __init__(self, dst, records: List[RawRecord]):
        self._dst = dst
        self._records = records[:]

    def __len__(self):
        return len(self._records)

    def __iter__(self):
        for record in iter(self._records):
            record = Record(self._dst, record)
            yield record

    def __str__(self):
        return f"(QuerySet: {self.count()} records)"

    __repr__ = __str__

    def __getitem__(self, index):
        return Record(self._dst, self._records[index])

    def chunks(self):
        """
        将当前 QuerySet 拆分成多个最大记录数为 10 的 QuerySet
        @return: iter[QuerySet] 迭代返回最大记录数为 10 的子 QuerySet
        """
        for chunk in chunks(self._records, MAX_WRITE_RECORDS_PRE_REQ):
            yield QuerySet(self._dst, chunk)

    def last(self):
        """
        获取最后一条记录
        @return: Record
        """
        return Record(self._dst, self._records[-1])

    def first(self):
        """
        获取第一条记录
        @return: Record
        """
        return Record(self._dst, self._records[0])

    def delete(self, delete_all=False) -> bool:
        """
        delete 批量删除当前记录集的所有记录，按 10 条一批次删除。记录集数量大于 10 条则报错
        @return:
        """
        if not delete_all and self.count() > MAX_WRITE_RECORDS_PRE_REQ:
            raise Exception('不能批量操作大于 10 条记录，请使用 chunks 方法，分批操作。')
        return self._dst.delete_records([rec.id for rec in self._records])

    def clone(self):
        """
        返回当前记录集的克隆副本
        @return: QuerySet
        """
        return QuerySet(self._dst, self._records[:])

    def update(self, **kwargs) -> bool:
        """
        dst.records.filter(title=None).update(status="Pending")
        """
        if self.count() > MAX_WRITE_RECORDS_PRE_REQ:
            raise Exception('不能批量操作大于 10 条记录，请使用 chunks 方法，分批操作')
        patch_update_records_data = []
        for record in iter(self._records):
            data = {"recordId": record.id, "fields": kwargs}
            patch_update_records_data.append(data)
        self._records = self._dst.update_records(patch_update_records_data)
        return True

    def count(self):
        """
        返回当前记录集的总记录数量
        @return: int
        """
        return len(self)

    def get(self, **kwargs):
        if kwargs:
            return self.filter(**kwargs).first()
        if self._records:
            return Record(self._dst, self._records[0])
        raise RecordDoesNotExist()

    def filter(self, *args, **kwargs):
        """
        songs = dst_songs.filter(artist="faye wong")
        songs = dst_songs.filter('{artist}="faye wong" OR {artist}="someone"')
        for song in songs:
            print(song.title)
        """
        if args and kwargs:
            #
            print()
        # if filter_func_or_key:
        #     # 如果是函数
        #     if isinstance(filter_func_or_key, FunctionType):
        #         if kwargs:
        #             raise Exception("function filter can't be used with key-value filter")
        kwargs = {
            trans_key(self._dst.field_key_map, k): v
            for k, v in kwargs.items()
        }

        def filter_record(record) -> bool:
            return all([
                record.id == v
                if k in ["recordId", "_id"] else record.data.get(k) == v
                for k, v in kwargs.items()
            ])

        found_records = list(filter(filter_record, self._records))
        return QuerySet(self._dst, found_records)



import unittest
import warnings

from vika import Vika
from . import TEST_API_BASE, TEST_API_TOKEN


class TestGetSpaces(unittest.TestCase):

    def setUp(self):
        warnings.simplefilter('ignore', ResourceWarning)
        vika = Vika(TEST_API_TOKEN)
        vika.set_api_base(TEST_API_BASE)
        self.vika = vika

    def test_spaces_all(self):
        spaces = self.vika.spaces.all()
        self.assertIsInstance(spaces, list)
        first_space = spaces[0]
        self.assertIsNotNone(first_space.name)
        self.assertIsInstance(first_space.name, str)


if __name__ == '__main__':
    unittest.main()



from typing import Dict, Any, List, Optional

from vika.types.node import NodeListItem, NodeDetail
from vika.types.space import SpaceListItem
from vika.types.record import RawRecord
from vika.types.view import MetaView
from vika.types.field import MetaField
from pydantic import BaseModel


class ResponseBase(BaseModel):
    """
    REST API 返回的统一响应体格式
    """
    code: int
    success: bool
    message: str
    data: Optional[Any]


Records = List[RawRecord]


class RecordsData(BaseModel):
    records: Records


class GETResponseBase(BaseModel):
    total: int
    pageNum: int
    pageSize: int


class GETRecordResponseData(GETResponseBase, RecordsData):
    pass


class GETRecordResponse(ResponseBase):
    """
    获取记录返回结果
    """
    data: GETRecordResponseData


class PatchRecordResponse(ResponseBase):
    """
    更新记录返回结果
    """
    data: RecordsData


class PostRecordResponse(ResponseBase):
    """
    创建记录返回结果
    """
    data: RecordsData


class DeleteRecordResponse(ResponseBase):
    """
    删除记录返回结果
    """
    pass


class UploadFileResponse(ResponseBase):
    """
    上传文件返回结果
    """
    data: Dict[str, Any]


# meta field
class GETMetaFieldResponseData(BaseModel):
    items: List[MetaField]

    class Config:
        # https://github.com/samuelcolvin/pydantic/issues/1250
        fields = {
            "items": "fields",  # 服务端返回的数据为 fields，此字段为 pydantic 保留字段。
        }


class GETMetaFieldResponse(ResponseBase):
    """
    获取 meta field 返回数据
    """
    data: GETMetaFieldResponseData


class PostMetaFieldResponseData(BaseModel):
    """ 创建字段后的返回数据结构

    """
    id: str
    name: str


class PostMetaFieldResponse(ResponseBase):
    """ 创建字段后的返回数据

    """
    data: PostMetaFieldResponseData


class DeleteFieldResponse(ResponseBase):
    """
    删除字段返回结果
    """
    pass


# meta view
class GETMetaViewResponseData(BaseModel):
    views: List[MetaView]


class GETMetaViewResponse(ResponseBase):
    """
    获取 meta view 返回数据
    """
    data: GETMetaViewResponseData


# space
class GETSpaceListResponseData(BaseModel):
    spaces: List[SpaceListItem]


class GETSpaceListResponse(ResponseBase):
    """
    获取「空间站列表」返回的数据
    """
    data: GETSpaceListResponseData


# node
class GETNodeListResponseData(BaseModel):
    nodes: List[NodeListItem]


class GETNodeListResponse(ResponseBase):
    """
    获取「文件列表」返回的数据
    """
    data: GETNodeListResponseData


class GETNodeDetailResponse(ResponseBase):
    """
    获取「文件详情」返回的数据
    """
    data: NodeDetail


class PostDatasheetMetaResponseData(BaseModel):
    """创建表格响应数据结构

    """
    id: str
    createdAt: int
    items: List[PostMetaFieldResponseData]

    class Config:
        # https://github.com/samuelcolvin/pydantic/issues/1250
        fields = {
            "items": "fields",  # 服务端返回的数据为 fields，此字段为 pydantic 保留字段。
        }


class PostDatasheetMetaResponse(ResponseBase):
    """创建表格响应数据

    """
    data: PostDatasheetMetaResponseData




import unittest
import warnings

from vika import Vika
from . import TEST_TABLE, TEST_API_BASE, TEST_API_TOKEN


class TestGetFields(unittest.TestCase):

    def setUp(self):
        warnings.simplefilter('ignore', ResourceWarning)
        vika = Vika(TEST_API_TOKEN)
        vika.set_api_base(TEST_API_BASE)
        self.dst = vika.datasheet(TEST_TABLE)

    def test_fields_all(self):
        fields = self.dst.fields.all()
        self.assertIsInstance(fields, list)
        primary_field = fields[0]
        self.assertTrue(primary_field.isPrimary)
        self.assertEqual(primary_field.name, 'title')
        self.assertEqual(primary_field.type, 'SingleText')

    def test_field_get(self):
        primary_field = self.dst.fields.get('title')
        self.assertTrue(primary_field.isPrimary)
        self.assertEqual(primary_field.name, 'title')
        self.assertEqual(primary_field.type, 'SingleText')


if __name__ == '__main__':
    unittest.main()



from vika.types.response import PostDatasheetMetaResponseData


class DatasheetManager:
    """

    """
    def __init__(self, spc: 'Space'):
        self.spc = spc

    def create(self, data) -> PostDatasheetMetaResponseData:
        """ 字段创建  https://vika.cn/developers/api/reference/#operation/create-datasheets

            :param dict data:  新建表格属性
            :return: 新建表格id、创建时间戳和字段id、name信息
            :raises ServerError: 服务端错误
            :raises ResponseBodyParserError: 解析响应体失败
            :raises Exception: 其他异常

            :example:
            >>> vika = Vika('YOUR_API_TOKEN')
            >>> req_data = {'name': 'table_name'}
             >>> dst_meta = vika.space('YOUR_SPACE_ID').datasheets.create(req_data)
        """
        resp = self.spc.create_datasheet(data)
        return resp.data



import unittest
import warnings
from vika import Vika
from . import TEST_TABLE, TEST_API_BASE, TEST_API_TOKEN, TEST_SPACE_ID


class TestCreateFields(unittest.TestCase):
    """VIKA Python SDK 创建字段测试类
        - 字段创建SDK测试
    """

    def setUp(self):
        """创建字段测试类 对象初始hook：
            - 与资源使用相关的警告忽略
            - 初始化Vika对象
            - 获取操作表
        """
        warnings.simplefilter('ignore', ResourceWarning)
        vika = Vika(TEST_API_TOKEN)
        vika.set_api_base(TEST_API_BASE)
        self.dst = vika.space(TEST_SPACE_ID).datasheet(TEST_TABLE)

    def test_field_create(self):
        """字段创建SDK测试

        """
        req_data = {'type': 'SingleText', 'name': '标题', 'property': {'defaultValue': '默认文本'}}
        self.field = self.dst.fields.create(req_data)
        self.assertIsNotNone(self.field.id)
        self.assertEqual(self.field.name, '标题')

    def tearDown(self):
        """创建字段测试类 对象销毁hook：
            - 删除测试创建的字段
        """
        self.dst.fields.delete(self.field.id)


if __name__ == '__main__':
    unittest.main()



import requests

from .const import API_BASE
from .datasheet import Datasheet
from .node import NodeManager
from .space import Space, SpaceManager
from .utils import get_dst_id


class Vika:
    def __init__(self, token: str, **kwargs):
        """
        @param token: API Token, 用户的开发者令牌
        @param kwargs:
            - api_base: api 接口地址，默认是 "https://vika.cn"
        """
        self.request = requests.session()
        self._auth(token)
        self._api_base = kwargs.get('api_base', API_BASE)

    def _auth(self, token):
        self.request.headers.update({"Authorization": f"Bearer {token}"})

    @property
    def api_base(self):
        """
        当前 API 基础地址，区分环境
        """
        return self._api_base

    def set_api_base(self, api_base):
        """
        设置 api_base
        @param api_base:
        @return:
        """
        self._api_base = api_base

    def space(self, space_id: str):
        """
        指定空间站资源
        @param space_id: 空间站 id
        @return: Space
        """
        return Space(self, space_id)

    @property
    def spaces(self):
        """
        空间站资源管理
        @return: SpaceManager
        """
        return SpaceManager(self)

    @property
    def nodes(self):
        """
        文件节点资源管理
        @return: NodeManager
        """
        return NodeManager(self)

    def datasheet(self, dst_id_or_url, **kwargs):
        """
        指定维格表资源
        @param dst_id_or_url: 维格表 ID 或者 维格表 URL
        @param kwargs:
            - field_key: 'id' or 'name' 按字段ID或字段名处理 records
            - field_key_map: 字段映射字典。参见：https://github.com/vikadata/vika.py#%E5%AD%97%E6%AE%B5%E6%98%A0%E5%B0%84
        @return:
        """
        dst_id = get_dst_id(dst_id_or_url)
        return Datasheet(self, dst_id, records=[], **kwargs)



from vika.node import NodeManager
from vika.utils import get_dst_id, handle_response
from vika.datasheet import Datasheet, DatasheetManager
from vika.types.response import PostDatasheetMetaResponse
from urllib.parse import urljoin


class Space:
    def __init__(self, vika: 'Vika', space_id: str):
        self.vika = vika
        self.id = space_id

    @property
    def nodes(self):
        return NodeManager(self.vika, space_id=self.id)

    @property
    def datasheets(self):
        return DatasheetManager(self)

    def datasheet(self, dst_id_or_url, **kwargs):
        """
        指定维格表资源
        @param dst_id_or_url: 维格表 ID 或者 维格表 URL
        @param kwargs:
            - field_key: 'id' or 'name' 按字段ID或字段名处理 records
            - field_key_map: 字段映射字典。参见：https://github.com/vikadata/vika.py#%E5%AD%97%E6%AE%B5%E6%98%A0%E5%B0%84
        @return:
        """
        dst_id = get_dst_id(dst_id_or_url)
        return Datasheet(self.vika, dst_id, spc_id=self.id, **kwargs)

    def create_datasheet(self, data) -> PostDatasheetMetaResponse:
        """ 表格创建

            :param dic data: api请求体，结构：{'name': 'table_name'}
            :return: 创建表格响应数据
            :raises ServerError: 服务端错误
            :raises ResponseBodyParserError: 解析响应体失败
            :raises Exception: 其他异常
        """
        if self.id is None:
            raise Exception('maybe: vika.datasheet("dst_id") => vika.space("spc_id").datasheet("dst_id")')
        api_endpoint = urljoin(self.vika.api_base,
                               f"/fusion/v1/spaces/{self.id}/datasheets")
        resp = self.vika.request.post(api_endpoint, json=data)
        return handle_response(resp, PostDatasheetMetaResponse)



from typing import List

from vika.const import MAX_GET_RECORDS_PRE_REQ, MAX_WRITE_RECORDS_PRE_REQ
from vika.datasheet.query_set import QuerySet
from vika.datasheet.record import Record
from vika.exceptions import RecordDoesNotExist
from vika.types import GETRecordResponse
from vika.utils import query_parse, trans_data


class RecordManager:

    def __init__(self, dst: 'Datasheet'):
        self._dst = dst
        self._fetched_with = None
        self._fetched_by = None

    def bulk_create(self, data) -> List[Record]:
        """
        批量创建记录，每个请求只能创建 10 条记录 dst.records.bulk_create([{"标题": "hello vika"}])
        @param data: 记录对象列表 [{ "fieldKey": fieldValue }, { "fieldKey": fieldValue2 }]
        @return: List[Record]
        """
        if len(data) > MAX_WRITE_RECORDS_PRE_REQ:
            raise Exception(f'单个请求创建记录数量不得大于 {MAX_WRITE_RECORDS_PRE_REQ} 条')
        resp = self._dst.create_records(data)
        return [Record(self._dst, record) for record in resp.data.records]

    def bulk_update(self, data) -> List[Record]:
        """
        批量更新记录，和原始的 API 请求一样
        @param data: 记录对象列表 [{ "recordId": "recxxxx", "fields": {"fieldKey": fieldValue} }]
        @return Boolean
        """
        if len(data) > MAX_WRITE_RECORDS_PRE_REQ:
            raise Exception(f'单个请求更新记录数量不得大于 {MAX_WRITE_RECORDS_PRE_REQ} 条')
        for record in data:
            record['fields'] = trans_data(self._dst.field_key_map,
                                          record['fields'])
        updated_records = self._dst.update_records(data)
        return [Record(self._dst, record) for record in updated_records]

    def create(self, data) -> Record:
        """
        创建一条记录 dst.records.create({"标题": "hello vika"})
        @param data: dict {"fieldKey": fieldValue}
        @return: Record
        """
        resp = self._dst.create_records(data)
        if resp.success:
            records = resp.data.records
            return Record(self._dst, records[0])
        raise Exception(resp.message)

    def all(self, **kwargs):
        """
        按查询条件过滤记录，参见获取记录 query params https://vika.cn/developers/api-reference#operation/get-records
        @param kwargs:
            - viewId: 'viewId1', 视图ID。默认为维格表中第一个视图。请求会返回视图中经过视图中筛选/排序后的结果，可以搭配使用fields参数过滤不需要的字段数据
            - sort: [{ '列名称或者 ID': 'asc' }] 对指定维格表的记录进行排序。由多个“排序对象”组成的数组。支持顺序：'asc' 和 逆序：'desc'。注：此参数指定的排序条件将会覆盖视图里的排序条件。
            - recordIds: ['recordId1', 'recordId2'],recordIds 数组。如果附带此参数，则返回参数中指定的records数组。 返回值按照传入数组的顺序排序。此时无视筛选、排序。无分页，每次最多查询 1000 条
            - fields: ['标题', '详情', '引用次数'], 指定要返回的字段（默认为字段名, 也可以通过 fieldKey 指定为字段 Id）。如果附带此参数，则返回的记录合集将会被过滤，只有指定的字段会返回。
            - filterByFormula: '{引用次数} >  0', 使用公式作为筛选条件，返回匹配的记录，访问 https://vika.cn/help/tutorial-getting-started-with-formulas/ 了解公式使用方式
            - maxRecords: 5000, 限制返回记录的总数量。如果该值小于表中实际的记录总数，则返回的记录总数会被限制为该值。
            - cellFormat: 'json', 单元格值类型，默认为 'json'，指定为 'string' 时所有值都将被自动转换为 string 格式。
            - fieldKey: 'name', 指定 field 的查询和返回的 key。默认使用列名  'name' 。指定为 'id' 时将以 fieldId 作为查询和返回方式（使用 id 可以避免列名的修改导致代码失效问题）
        @return:
        """
        _fieldKey = kwargs.get("fieldKey")
        if _fieldKey and _fieldKey != self._dst.field_key:
            # TODO: logger warning
            print(
                f'It seems that you set field_key when init datasheet, all(filedKey="{_fieldKey}") wont work'
            )
        kwargs.update(fieldKey=self._dst.field_key)
        if 'pageSize' in kwargs or 'pageNum' in kwargs:
            resp: GETRecordResponse = self._dst.get_records(**kwargs)
            if resp.success:
                records = resp.data.records
            else:
                print(f"[{self._dst.id}] fetch data fail\n {resp.message}")
                records = []
        else:
            records = self._dst.get_records_all(**kwargs)
        return QuerySet(self._dst, records)

    def get(self, *args, **kwargs):
        """
        查询出符合条件的单条记录，适合使用唯一标识的字段做查询
        1. 指定 record id 查询记录
        dst_books.records.get("recxxxxxx")
        2. 按条件查询记录
        dst_books.records.get(ISBN="9787506341271")
        @param args:
        @param kwargs:
        @return:
        """
        # 按 id 查找
        if args:
            record_id = args[0]
            kwargs = {"recordIds": [record_id]}
        elif kwargs:
            query_formula = query_parse(self._dst.field_key_map, **kwargs)
            kwargs = {"filterByFormula": query_formula}
        resp: GETRecordResponse = self._dst.get_records(**kwargs)
        if resp.data.records:
            return Record(self._dst, resp.data.records[0])
        raise RecordDoesNotExist()

    def get_or_create(self, defaults=None, **kwargs):
        """
        按给定的 kwargs 条件查找单条记录，如果没有符合的记录，则创建记录。
        eg: dst.records.get_or_create(defaults={"done":False}, task="今天早起") 如果找不到 「task 为「今天早起」的记录，则创建一条，且的「done」是「False」
        @param defaults 字典（字段key,字段值），当记录不存在时，创建记录的默认值。
        @param kwargs 查询条件
        @return (record,created) 返回记录和是否是创建
        """
        created = False
        record = None
        data = {}
        try:
            record = self.get(**kwargs)
        except RecordDoesNotExist:
            created = True
            if kwargs:
                data.update(kwargs)
            if defaults:
                data.update(defaults)
            record = self.create(data)
        return (record, created)

    def update_or_create(self, defaults=None, **kwargs):
        """
        按给定的 kwargs 条件查找单条记录，如果没有符合的记录，则创建记录。如果找到了，就以 defaults 中的值更记录。
        @param defaults 字典（字段key,字段值），当记录不存在时，创建记录的默认值。
        @param kwargs 查询条件
        @return (record,created) 返回记录和是否是创建
        """
        created = False
        record = None
        data = {}
        try:
            record = self.get(**kwargs).update(defaults)
        except RecordDoesNotExist:
            created = True
            if kwargs:
                data.update(kwargs)
            if defaults:
                data.update(defaults)
            record = self.create(data)
        return (record, created)

    def filter(self, **kwargs):
        """
        通过查询条件，查询出符合条件的记录集
        songs = dst_songs.records.filter(artist="faye wong")
        for song in songs:
            print(song.title)
        @param kwargs:
        @return: QuerySet
        """
        # 直接通过 filter 调用时候，将 filter 查询参数转化为 filterByFormula，使用服务端计算结果
        records = self._query_records(**kwargs)
        return QuerySet(self._dst, records)

    def _query_records(self, **kwargs):
        # 将查询条件转化为 filterByFormula， 利用服务端计算查询记录集
        query_formula = query_parse(self._dst.field_key_map, **kwargs)
        kwargs = {
            "filterByFormula": query_formula,
            "pageSize": MAX_GET_RECORDS_PRE_REQ
        }
        return self._dst.get_records_all(**kwargs)



from typing import Dict, Any
from pydantic import BaseModel


# 避免和 Record class 重名
class RawRecord(BaseModel):
    """
    REST API 返回的 record 原始类型
    """

    id: str
    data: Dict[str, Any]
    # createdAt: int
    # updatedAt: int

    class Config:
        # https://github.com/samuelcolvin/pydantic/issues/1250
        fields = {
            "data": "fields",  # 服务端返回的数据为 fields，此字段为 pydantic 保留字段。
            "id": "recordId",  # 服务端返回的数据为 recordId，这里映射成 id
        }



class DatasheetDoesNotExist(Exception):
    """
    数表不存在
    """
    pass


class RecordDoesNotExist(Exception):
    """
    记录不存在
    """
    pass


class FieldDoesNotExist(Exception):
    """
    字段不存在
    """

    pass


class RecordWasDeleted(Exception):
    """
    主动删除记录后，再访问此记录的属性会报错
    """

    pass


class ErrorFieldKey(Exception):
    """
    错误的 fieldKey
    """

    pass


class ErrorSortParams(Exception):
    """
    错误的排序规则
    """
    pass


class UploadFileError(Exception):
    """
    上传附件失败
    """
    pass


class JSONDecodeError(Exception):
    """
    JSON 解析错误
    """
    pass


class ResponseBodyParserError(Exception):
    """
    解析响应体失败
    """
    pass


class ServerError(Exception):
    """
    服务器错误
    """
    pass



import time
import unittest
import warnings

from vika import Vika

from . import TEST_API_BASE, TEST_API_TOKEN, TEST_TABLE


class TestDeleteRecords(unittest.TestCase):

    def setUp(self):
        warnings.simplefilter('ignore', ResourceWarning)
        vika = Vika(TEST_API_TOKEN)
        vika.set_api_base(TEST_API_BASE)
        self.dst = vika.datasheet(TEST_TABLE)

    def test_record_delete(self):
        record = self.dst.records.get(title="无人生还")
        time.sleep(1 / 5)
        r = record.delete()
        self.assertTrue(r)

        time.sleep(1 / 5)
        record_id = "recnotexist"
        with self.assertRaises(Exception) as e:
            self.dst.delete_records([record_id])
        self.assertIsNotNone(e)

    def tearDown(self):
        time.sleep(1)
        self.dst.records.create({"title": "无人生还"})


if __name__ == "__main__":
    unittest.main()



import unittest
import warnings
from vika import Vika
from . import TEST_API_BASE, TEST_API_TOKEN, TEST_SPACE_ID


class TestCreateFields(unittest.TestCase):
    """VIKA Python SDK 创建表格测试类
        - 表格创建SDK测试
    """

    def setUp(self):
        """创建表格测试类 对象初始hook：

        """
        warnings.simplefilter('ignore', ResourceWarning)
        vika = Vika(TEST_API_TOKEN)
        vika.set_api_base(TEST_API_BASE)
        self.spc = vika.space(TEST_SPACE_ID)

    def test_datasheet_create(self):
        """表格创建SDK测试

        """
        req_data = {'name': 'table_name'}
        self.datasheet = self.spc.datasheets.create(req_data)
        self.assertIsNotNone(self.datasheet.id)

    def tearDown(self):
        """创建表格测试类 对象销毁hook：

        """


if __name__ == '__main__':
    unittest.main()




import unittest
import warnings

from vika import Vika

from .env import TEST_API_BASE, TEST_API_TOKEN, TEST_SPACE_ID, TEST_FOLDER_ID


class TestGetNodes(unittest.TestCase):

    def setUp(self):
        warnings.simplefilter('ignore', ResourceWarning)
        vika = Vika(TEST_API_TOKEN)
        vika.set_api_base(TEST_API_BASE)
        self.vika = vika

    def test_space_root_nodes_all(self):
        space_root_nodes = self.vika.space(TEST_SPACE_ID).nodes.all()
        self.assertIsInstance(space_root_nodes, list)
        first_node = space_root_nodes[0]
        self.assertEqual(first_node.type, 'Folder')

    def test_get_node_detail(self):
        node = self.vika.nodes.get(TEST_FOLDER_ID)
        self.assertEqual(node.id, TEST_FOLDER_ID)
        self.assertIsNotNone(node.children)


if __name__ == '__main__':
    unittest.main()



import unittest
import warnings

from vika import Vika
from . import TEST_TABLE, TEST_API_BASE, TEST_API_TOKEN


class TestGetRecords(unittest.TestCase):

    def setUp(self):
        warnings.simplefilter('ignore', ResourceWarning)
        vika = Vika(TEST_API_TOKEN)
        vika.set_api_base(TEST_API_BASE)
        self.dst = vika.datasheet(TEST_TABLE)

    def test_record_count(self):
        self.assertEqual(self.dst.records.all().count(), 1)

    def test_record_filter_get(self):
        self.assertEqual(
            self.dst.records.filter(title="无人生还").get().title, "无人生还")

    def test_record_all(self):
        # 不存在的视图返回空记录
        self.assertEqual(
            self.dst.records.all(viewId="viw6oKVVbMynt").count(), 0)

    def test_get_or_create(self):
        # 获取记录
        record, created = self.dst.records.get_or_create(title="无人生还")
        self.assertFalse(created)

        # 创建记录
        record, created = self.dst.records.get_or_create(title="不存在的记录")
        record.delete()
        self.assertEqual(record.title, "不存在的记录")
        self.assertTrue(created)

    # FXIME: 现在 rest api 返回有问题，先不加这条
    # def test_record_all_with_params(self):
    #     # 不存在的分页返回空记录
    #     self.assertEqual(self.dst.records.all(pageNum=2).count(), 0)


if __name__ == '__main__':
    unittest.main()
